# Amazon DAX Client for JavaScript

DAX is a DynamoDB-compatible caching service that enables you to benefit from fast in-memory performance for demanding applications.

This client library provides access from NodeJS to DAX.

## Installing
The Amazon DAX client only runs from NodeJS, and can be installed using npm:
```sh
npm install amazon-dax-client
```

## Usage and Getting Started
You can follow the Getting Started tutorial at:

https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DAX.client.sample-app.html

To quickly use DAX, replace `AWS.DynamoDB` with `AmazonDaxClient`:

```javascript
const AmazonDaxClient = require('amazon-dax-client');

// Replace this ...
var ddb = new AWS.DynamoDB({region: region});
/// with this ...
var dax = new AmazonDaxClient({endpoints: [endpoint], region: region});

// If using AWS.DynamoDB.DocumentClient ...
var doc = new AWS.DynamoDB.DocumentClient({service: dax});
```

The DAX endpoint can be found the AWS console or by using `aws dax describe-clusters` from the command line.

## Getting Help
Please use these community resources for getting help.

 * Ask a question on [StackOverflow](https://stackoverflow.com/) and tag it with `amazon-dynamodb-dax`
 * Ask a question on [the AWS DynamoDB forum](https://forums.aws.amazon.com/forum.jspa?forumID=131&start=0)
 * Open a support ticket with [AWS Support](https://console.aws.amazon.com/support/home#/)


## Changes

#### 1.2.4
* Minor bug fixes.

#### 1.2.3
* Retry prefer to choose a different node.
* Adjust exponential back off delay and strategy.

#### 1.2.2
* Fix unrecoverable request timeout after DAX node restart.
* Fix connection timeout during DAX server restart.
* Allow retry for IO exceptions.
* Remove verbose connection error logs.
* Fix empty string encoding bug.

#### 1.2.1
* Bug fix in TransactionGetItems Api to return Key element correctly.

#### 1.2.0
* Support for DynamoDB Transaction Apis.
* Fix excessive background endpoint refresh calls.

#### 1.1.4
* Fix connection leaks on timeout.
* Support configurable requestTimeout. (Default: 60s)

#### 1.1.3
* Fix transient error on tube cleanup
* Fix UpdateItem result parsing with a subset of attributes changed
* Increase default connectTimeout and make it configurable

#### 1.1.2
* Fix socket leaks on validation errors
* Fix SocketTimeout caused by connection establishment
* Fix issue with update response if the item is not changed
* Fix error decoding BatchWrite UnprocessedItems.

#### 1.1.1
* Fix socket leaks on errors

#### 1.1.0
* Brand new decoder for improved performance on large results
* Improved connection/timer management to allow clean exit on Lambda

#### 1.0.2
* Improved connection/timer management to allow clean exit on Lambda

#### 1.0.1
* Initial release

